function [is, PiEst, Gamma1Est, Gamma2Est, Omega, psi]  = computeIS2(P,beta,p)

T = size(P,2);
n = size(P,1);

ret = [zeros(n,1),diff(P')'];

% OLS estimation of VECM

Y1 = ret(1,p:end)';
Y2 = ret(2,p:end)';
C = (beta'*P(:,p-1:end-1))';
X1 = ret(:,p-1:end-1)';
X2 = ret(:,p-2:end-2)';

auxEst1 = regress(Y1,[C,X1,X2]);
alphaEst(1,1) = auxEst1(1); Gamma1Est(1,1) = auxEst1(2); Gamma1Est(1,2) = auxEst1(3); Gamma2Est(1,1) = auxEst1(4); Gamma2Est(1,2) = auxEst1(5);

auxEst2 = regress(Y2,[C,X1,X2]);
alphaEst(2,1) = auxEst2(1); Gamma1Est(2,1) = auxEst2(2); Gamma1Est(2,2) = auxEst2(3); Gamma2Est(2,1) = auxEst2(4); Gamma2Est(2,2) = auxEst2(5);

PiEst = alphaEst*beta';

resEst = nan(n,T);
for t = p:T
    resEst(:,t) = ret(:,t) - PiEst*P(:,t-1) - Gamma1Est*ret(:,t-1) - Gamma2Est*ret(:,t-2);
end

% Computation of IS through the Beveridge-Nelson decomposition

orthAlpha = null(alphaEst');
orthBeta = null(beta');

psi = (orthBeta/(orthAlpha'*(eye(n)-Gamma1Est-Gamma2Est)*orthBeta))*orthAlpha';
%psi = (orthBeta/(orthAlpha'*(eye(n))*orthBeta))*orthAlpha';
psi = psi(1,:);

Omega = cov(resEst(:,p+1:end)');
varBN = psi*Omega*psi';
F = (chol(Omega))';
aux = psi*F;

is = nan(n,1);
for i = 1:n
    is(i) = aux(i)^2/varBN;
end

