clear all;
close all;
clc;

addpath(genpath(pwd));

%% General settings

specData;

HY = 1;
KEM = 1;
LLKEM = 1;

plotKEMFig = 0;
plotLLKEMFig = 0;

nIterEM = 300;         % maximum n. of iterations
tolEM = 1e-5;          % min likelihood changing tolerance
showPlotEM = false;

nIterLLEM = 2000;         % maximum n. of iterations
tolLLEM = 1e-8;          % min likelihood changing tolerance
showPlotLLEM = false;

%% Preprocessing

if preprocessing
    
    preProc;
    
else
    
    str1 = sprintf('%s2ndRound/PP.mat',groupType);
    str2 = sprintf('%s2ndRound/timeMaster.mat',groupType);
    load(str1); load(str2);
    d = size(PP,2);
    
end

%% Processing

nDaysToAnalyze = numel(timeMaster);

cConv = nan(nDaysToAnalyze,1);
cConvLL = nan(nDaysToAnalyze,1);
logLik = nan(nDaysToAnalyze,1);
logLikLL = nan(nDaysToAnalyze,1);
LEst = nan(nDaysToAnalyze,d,d);
QLLKEM = nan(nDaysToAnalyze,d,d);
RLLKEM = nan(nDaysToAnalyze,d,d);
QKEM = nan(nDaysToAnalyze,d,d);
RKEM = nan(nDaysToAnalyze,d,d);

BICkem = nan(1,nDaysToAnalyze);
AICkem = nan(1,nDaysToAnalyze);
BICllem = nan(1,nDaysToAnalyze);
AICllem = nan(1,nDaysToAnalyze);

k1 = d+d*(d+1)/2;    %n. of parameters to be estimated in KEM
k2 = k1 + d^2;       %n. of parameters to be estimated in LLKEM

% c = parcluster;
% c.NumWorkers = 8;
% parpool('local',8);

freq = 1;  % sampling frequency in seconds

for i = 1:nDaysToAnalyze
    
    fprintf('Elaborating %s...\n',datestr(timeMaster(i)));
    
    Pin = log(squeeze(PP(i,:,1:freq:end)));
    T = size(Pin,2);
    
    if HY
        try
            n1 = 1;
            n2 = 30;
            
            %Compute R using the two-scale estimator
            RHY = zeros(d);
            for p=1:d
                yIn = Pin(p,isnan(Pin(p,:))==0);
                nObs = sum(~isnan(Pin(p,:)));
                [~,auxRHY] = TSRV(yIn',n1,n2);
                RHY(p,p) = auxRHY;
            end
            
            RHY = RHY/(T-1);
            QCHY = HayashiYoshida(Pin',n1,n2);
            QCHY = frobproj(QCHY,1e-4);
            QHY = QCHY/(T-1); 
        end
    end
    
    if KEM
        try  
            % EM initialization
            Q0 = QHY;
            R0 = RHY;
            
            [~,~,auxQKEM,auxRKEM,cConv(i),logLPath] = EM(Pin,Q0,R0,nIterEM,tolEM,showPlotEM);
            logLik(i) = logLPath(end);
            BICkem(1,i) = -2*logLik(i) + k1*log(T*d);
            AICkem(1,i) = 2*k1 - 2*logLik(i);
            
            QKEM(i,:,:) = auxQKEM;
            RKEM(i,:,:) = auxRKEM;
            
        end
    end
    
    if LLKEM
        
        try
            % EM initialization
            C = [eye(d),zeros(d)];
            Q0 = [auxQKEM,zeros(d);zeros(d,2*d)];
            R0 = squeeze(auxRKEM);
            L = 0.02*ones(d);
            F0 = [eye(d)+L,-L;eye(d),zeros(d)];
            
            [~,~,auxQLLKEM,auxRLLKEM,FLLKEM,cConvLL(i),logLPath] = LLEM(Pin,Q0,R0,F0,C,nIterLLEM,tolLLEM,showPlotLLEM);
            
            logLikLL(i) = logLPath(end);
            BICllem(1,i) = -2*logLikLL(i) + k2*log(T*d);
            AICllem(1,i) = 2*k2 - 2*logLikLL(i);
            
            QLLKEM(i,:,:) = auxQLLKEM(1:d,1:d);
            RLLKEM(i,:,:) = auxRLLKEM;
            LEst(i,:,:) = FLLKEM(1:d,1:d) - eye(d);
            
            % Check stability
            [~,D] = eig(squeeze(LEst(i,:,:)));
            if any(abs(diag(D)) >= 1)
                disp('Estimated transition matrix is not stationary!');
            end
        end
    end
end

varListKEM = {'QKEM','RKEM','AICkem','BICkem'};
varNameKEM = 'results/KEM.mat';
save(varNameKEM, varListKEM{:})

varListLLEM = {'QLLKEM','RLLKEM','LEst','AICllem','BICllem'};
varNameLLEM = 'results/LLEM.mat';
save(varNameLLEM, varListLLEM{:})

