function [state, obs] = simModel(d,T,R,par,L,rho,probMiss,S0)

% Check Feller condition

if any((par.k).*(par.theta) <= 0.5*(par.s).^2)
    fprintf('Feller condition is not met!\n');
    state = nan;
    obs = nan;
    QSim = nan;
    return
end

%Allocate memory
W = randn(d,T);
W1 = randn(d,T-1);

nu   = nan(d,T);
RX = nan(d,T);
Qt = nan(d,d,T);

% Parameter settings
CC = 0.88929198;
AA = 0.75; 
BB = 0.25;
a=10; b=10;

k = par.k;
theta = par.theta;
s = par.s;

deltaT  = 1/T;
aux1    = (1:T)/T;

% Generate U-shape
sigmaU = (CC + AA*exp(-a*aux1) + BB*exp(-b*(1-aux1)));

%Initialization
RX(:,1) = zeros(d,1);
nu(:,1) = theta;
aux2 = sigmaU(1)*sqrt(nu(:,1));
Qt(:,:,1) = diag(aux2)*rho*diag(aux2);

% Simulate state-space model
for t=2:T
    nu(:,t)          = nu(:,t-1) + k.*(theta - nu(:,t-1))*deltaT + s.*sqrt(nu(:,t-1)*deltaT).*W1(:,t-1);
    aux2 = sigmaU(t)*sqrt(nu(:,t));
    Qt(:,:,t) = diag(aux2)*rho*diag(aux2);
    cholQ = chol(Qt(:,:,t))';
    RX(:,t) = L*RX(:,t-1) + cholQ*W(:,t-1);
  
end

state = cumsum([S0,RX(:,2:end)],2);

missIndex = rand(d,T)<probMiss;

chR = chol(R)';
noise = chR*randn(d,T);

obs = state + noise;
obs(missIndex) = nan;



