clear all;
close all;
clc;

addpath(genpath(pwd));

%% General settings

specSim;

save('results/QSim.mat','QSim');
save('results/LSim.mat','LSim');
save('results/RSim.mat','RSim');

HY = true;
LLKEM = true;
KEM = true;

%% Processing

LEst = nan(nSim,d,d); 
QEst = nan(nSim,d,d); 
REst = nan(nSim,d,d);

specBenchmarks;

for i = 1:nSim
    
    fprintf('Simulation n. %s\n',num2str(i));
    
    rng('shuffle');
    [S, P] = simModel(d,T,RSim,parCIR,LSim,corrSim,probMiss,S0);
    
    %%
    if HY
        
        % Compute R using the two-scale estimator
        RHY = zeros(d);
        for p=1:d
            yIn = P(p,isnan(P(p,:))==0);
            [~,auxRHY]=TSRV(yIn',n1,n2);
            RHY(p,p) = auxRHY;
        end
        RHY = RHY/(T-1);
        
        % Compute Q using Hayashi-Yoshida estimator
        QHY = HayashiYoshida(P',n1,n2);
        QHY = QHY/(T-1);
        
        RHY = frobproj(RHY,1e-4);
        QHY = frobproj(QHY,1e-4);
      
    end
    
    %%
    if KEM
        
        specEMsim;
        [xF,xS,QKEM,RKEM,~,~] = EMsim(P,Q0(1:d,1:d),R0,nIter,tol,trueQ(1:d,1:d),trueR,showPlot);
    end
    
    %%
    if LLKEM
        
        specLLEMsim;
        [~,~,QLLKEM,RLLKEM,FLLKEM,~,~,~,~] = LLEMsim(P,Q0,R0,F0,C,nIter,tol,trueQ,trueR,trueF,showPlot);
        
        LEst(i,:,:) = FLLKEM(1:d,1:d)-eye(d);
        QEst(i,:,:) = QLLKEM(1:d,1:d);
        REst(i,:,:) = RLLKEM;
        
        save('results/QEst.mat','QEst');
        save('results/LEst.mat','LEst');
        save('results/REst.mat','REst');
        
    end
        
end
    
    