
nSim = 1000;                                                    % n. of simulations
T = 6.5*60*60+1;                                                % n. of time steps
d = 2;                                                          % n. of assets

stnRatioSim = 1*ones(d,1);                                      % signal-to-noise ratio (varSignal/varNoise)
sigmaQBar = sqrt([0.1; 0.2]/10);                                % average model volatility 
probMiss = [0;0];                                               % average duration
sigmaRSim = sigmaQBar./sqrt(stnRatioSim);                       % observation noise volatility

Y = eye(d);                                                     % observation noise correlation
RSim = diag(sigmaRSim)*Y*diag(sigmaRSim);                       % observation noise covariance

corrSim = [1, 0.4; 0.4, 1];                                     % model innovation correlation 
QSim = diag(sigmaQBar)*corrSim*diag(sigmaQBar);

%% CIR parameters

parCIR.k = 10*ones(d,1);
parCIR.theta = sigmaQBar.^2;
parCIR.s = 0.1*ones(d,1);
parCIR.leverage = 0.01*ones(d,1);

%% Transition matrix

LSim = 0*ones(d); 
LSim(1,1) = 0.1;
LSim(2,2) = 0.1;
LSim(1,2) = 0.5;
LSim(2,1) = 0.3;

[~,D] = eig(LSim);
if any(abs(diag(D)) >= 1)
    disp('The simulated process is not stationary!');
end

%% Other settings

S0 = 50*ones(d,1);                                              % initial log-price
assetSet = {'asset 1', 'asset 2'};
save(strcat(pwd,'/results/assetSet.mat'),'assetSet');